;; Maintain the members participating in each forum.
;; Copyright (C) 2001  Neal H Walfield  <neal@cs.uml.edu>
;;
;; This file is part of the Deride
;;
;; Deride is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;; 
;; Deride is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with the GNU Hurd; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(define-module (forum))

;; An alist forum names pointing to a list of user objects.  Beyond
;; being consistent between references, objects are specified for the
;; purposes of this module.
(define forum-list '())

;; USER joined FORUM.  Add USER to the user list if not already
;; present.
(define-public (forum-user-joined forum user)
  (set! forum (string-upcase forum))
  (let ((user-list (assoc forum forum-list)))
    (if user-list
	;; The forum exists.
	(if (not (member user user-list))
	    ;; But the user has not yet been added.
	    (set! forum-list
		  (assoc-set! forum-list forum (cons user (cdr user-list)))))
	;; This is first user in the forum.
	(set! forum-list (acons forum (list user) forum-list)))))

;; Get the user list for FORUM.
(define-public (forum-user-list forum)
  (let ((user-list
	 (assoc (string-upcase forum) forum-list)))
    (if user-list
	(cdr user-list)
	'())))

;; Return #f if USER is not present in FORUM, otherwise not #f.
(define-public (forum-user-present forum user)
  (member user (forum-user-list forum)))

;; A user has parted a forum.
(define-public (forum-user-parted forum user)
  (set! forum (string-upcase forum))
  (let ((user-list (assoc forum forum-list)))
    (if user-list
	(begin (set! user-list (delete user (cdr user-list)))
	       (assoc-set! forum-list forum user-list)))))

;; A user has quit, i.e. parted all forums.
(define-public (forum-user-quit user)
  (let ((loop (lambda (list)
		forum-user-parted ((car list) user)
		(if (cdr list)
		    (loop (cdr list))))))
    (loop (forum-list))))